// ---------------------------------
// Filename      : RTCDS130x.cpp
// Author        : Sven Maerivoet
// Last modified : 11/11/2018
// Target        : C++ (Arduino UNO)
//
// All rights reserved.
// ---------------------------------

/*
  Copyright 2017, 2018 Sven Maerivoet

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

#include <RTCDS130x.h>

void DateTimeContainer::print()
{
  Serial.print(fDay, DEC);
  Serial.print(F("/"));
  Serial.print(fMonth, DEC);
  Serial.print(F("/"));
  Serial.print(fYear, DEC);
  Serial.print(F(" "));
  Serial.print(fHour, DEC);
  Serial.print(F(":"));
  Serial.print(fMinute, DEC);
  Serial.print(F(":"));
  Serial.print(fSecond, DEC);
  Serial.println();
}

/***************
 * CONSTRUCTOR *
 ***************/

RTCDS130x::RTCDS130x(ERTCType rtcType)
{
	fRTCType = rtcType;
	fInitialised = false;
	fPinsSpecified = (rtcType == kDS1307);
}

/******************
 * PUBLIC METHODS *
 ******************/

void RTCDS130x::setDS1302Pins(int clkPin, int datPin, int rstPin)
{
	if (fRTCType == kDS1302) {
	  fCLKPin = clkPin;
		fIOkPin = datPin;
		fCEPin = rstPin;
		fPinsSpecified = true;
	}
	else if (fRTCType == kDS1307) {
		fatalError(F("RTCDS130x: The DS1307 uses pins A4 (SDA, serial data) and A5 (SCL, serial clock) by default!"));
	}
}

void RTCDS130x::initialise()
{
	if (!fPinsSpecified) {
		fatalError(F("RTCDS130x: No pins specified!"));
	}

	if (fRTCType == kDS1302) {
  	fRTCDS1302 = new DS1302RTC(fCEPin, fIOkPin, fCLKPin);

	  if (fRTCDS1302->haltRTC()) {
	    fatalError(F("RTCDS130x: The DS1302 is stopped. Set the time manually!"));
	  }
	  // if (!fRTCDS1302->writeEN()) { The DS1302 is write protected (this is normal).

	  // setSyncProvider() causes the Time library to synchronise with the
	  // external RTC by calling RTC.get() every five minutes by default.
	  setSyncProvider(fRTCDS1302->get);

	  if (timeStatus() != timeSet) {
	    fatalError(F("RTCDS130x: Error: RTC not available!"));
	  }
	}
	else if (fRTCType == kDS1307) {
		fRTCDS1307 = new RTC_DS1307();
	  if (!fRTCDS1307->begin()) {
	    fatalError(F("RTCDS130x: Error: RTC not available!"));
	  }
	}

	fInitialised = true;
}

DateTimeContainer RTCDS130x::getDateTime()
{
	assertInitialised();

	DateTimeContainer dt;

	if (fRTCType == kDS1302) {
		time_t t = now();
		dt.fYear = year(t);
		dt.fMonth = month(t);
	  dt.fDay = day(t);
	  dt.fHour = hour(t);
	  dt.fMinute = minute(t);
	  dt.fSecond = second(t);
	}
	else {
		DateTime dateTime = fRTCDS1307->now();
		dt.fYear = dateTime.year();
		dt.fMonth = dateTime.month();
	  dt.fDay = dateTime.second();
	  dt.fHour = dateTime.hour();
	  dt.fMinute = dateTime.minute();
	  dt.fSecond = dateTime.second();
	}

	return dt;
}

void RTCDS130x::setDS1307ToNow(float compilationAndUploadTimeSec)
{
	if (fRTCType == kDS1307) {
 		fRTCDS1307->adjust(DateTime(F(__DATE__),F( __TIME__)) + compilationAndUploadTimeSec);
 	}
 	else {
		fatalError("RTCDS130x::setDS1307ToNow() is only applicable to the DS1307.");
 	}
}

/*******************
 * PRIVATE METHODS *
 *******************/

void RTCDS130x::assertInitialised()
{
	if (!fInitialised) {
		fatalError(F("RTCDS130x: RTC DS130x not initialised!"));
  }
}

void RTCDS130x::fatalError(String message)
{
	Serial.println(message);
	Serial.println(F("RTCDS130x: Execution halted."));
	while (true) {
  }
}
